@@ -4,7 +4,7 @@
 # Copyright 2010 - 2011 K. Richard Pixley.
 # See LICENSE for details.
 #
-# Time-stamp: <01-Feb-2011 12:13:42 PST by rich@noir.com>
+# Time-stamp: <07-Feb-2011 15:16:36 PST by rich@noir.com>
 
 """
 Elffile is a library which reads and writes `ELF format object files
@@ -30,6 +30,7 @@ import functools
 import io
 import mmap
 import operator
+import os
 import struct
 
 import coding
@@ -76,7 +77,7 @@ def open(name=None, fileobj=None, map=None, block=None):
         map = mmap.mmap(fileobj.fileno(), 0, mmap.MAP_SHARED, mmap.PROT_READ)
 
     elif name:
-        fileobj = io.open(name, 'rb')
+        fileobj = io.open(os.path.normpath(os.path.expanduser(name)), 'rb')
 
     else:
         assert False
@@ -257,6 +258,7 @@ class ElfFileIdent(StructBase):
                 and self.osabi == other.osabi
                 and self.abiversion == other.abiversion)
 
+    close_enough = __eq__
 
     def _list_encode(self):
         return (self.__class__.__name__,
@@ -687,30 +689,31 @@ class ElfFile(StructBase):
         if not isinstance(other, self.__class__):
             return False
 
-        if (self.fileIdent != other.fileIdent
-            or self.fileHeader != other.fileHeader):
+        if ((not self.fileIdent.close_enough(other.fileIdent))
+            or (not self.fileHeader.close_enough(other.fileHeader))):
             return False
 
         # FIXME: need to handle order independence
         for this, that in zip(self.sectionHeaders, other.sectionHeaders):
-            if this.name in [
+            if (this.name in [
                 '.ARM.attributes',
                 '.ARM.exidx',
                 '.ARM.extab',
                 '.comment',
                 '.debug_aranges',
                 '.debug_frame',
-                '.debug_info',
-                '.debug_line',
+                '.debug_info',    # x86_64 linux dyn
+                '.debug_line',    # arm debug lines contain file names
                 '.debug_loc',
                 '.debug_pubnames',
                 '.debug_ranges',
-                '.debug_str',
+                '.debug_str',           # x86_64 linux rela
                 '.note.GNU-stack',
+                '.note.gnu.build-id',   # x86_64 linux dyn
                 '.rel.ARM.exidx',
                 '.rel.debug_aranges',
                 '.rel.debug_frame',
-                '.rel.debug_info',
+                '.rel.debug_info',      # x86_64 linux rela
                 '.rel.debug_line',
                 '.rel.debug_pubnames',
                 '.rel.text',
@@ -719,10 +722,14 @@ class ElfFile(StructBase):
                 '.shstrtab',
                 '.strtab',
                 '.symtab',
-                ]:
+                ]
+                or this.type == SHT.byname['SHT_NOBITS'].code # Not sure what this is or why it differs
+                ):
                 continue
 
             if not this.close_enough(that):
+                import sys
+                print('section({0}) not close enough to section({1})'.format(this.name, that.name), file=sys.stdout)
                 return False
 
         return True
@@ -888,6 +895,21 @@ class ElfFileHeader(StructBase):
                 and self.shnum == other.shnum
                 and self.shstrndx == other.shstrndx)
 
+    def close_enough(self, other):
+        return (isinstance(other, self.__class__)
+                and self.type == other.type
+                and self.machine == other.machine
+                and self.version == other.version
+                and self.entry == other.entry
+                and self.phoff == other.phoff
+                and self.flags == other.flags
+                and self.ehsize == other.ehsize
+                and self.phentsize == other.phentsize
+                and self.phnum == other.phnum
+                and self.shentsize == other.shentsize
+                and self.shnum == other.shnum
+                and self.shstrndx == other.shstrndx)
+
     def __repr__(self):
         return ('<{0}@{1}: type={2}, machine={3}, version={4},'
                 ' entry={5}, phoff={6}, shoff={7}, flags={8},'
@@ -1252,6 +1274,19 @@ class ElfSectionHeader(StructBase):
                 and self.entsize == other.entsize
                 and self.content == other.content)
 
+    def close_enough(self, other):
+        return (isinstance(other, self.__class__)
+                and self.nameoffset == other.nameoffset
+                and self.type == other.type
+                and self.flags == other.flags
+                and self.addr == other.addr
+                and self.section_size == other.section_size
+                and self.link == other.link
+                and self.info == other.info
+                and self.addralign == other.addralign
+                and self.entsize == other.entsize
+                and self.content == other.content)
+
     def __repr__(self):
         # FIXME: I wish I could include the first few bytes of the content as well.
         return ('<{0}@{1}: name=\'{2}\', type={3},'
